#pragma once

#include <SFGUI/Config.hpp>
#include <SFGUI/FastDelegate.hpp>

#include <map>

namespace sfg {

/** Widget signal.
 * Calls a function if something interesting is happening in a widget. Signals
 * can be connected to multiple endpoints. An endpoint may be a free or member
 * function.
 *
 * For free functions, just pass the function's pointer to Connect(). For
 * member functions (methods) specify the class, method name and object.
 * Examples:\n
 * \code
 * widget->OnClick.Connect( &my_callback ); // Free function binding.
 * widget->OnClick.Connect( &MyClass::MyCallback, this ); // Method binding.
 * \endcode
 *
 */
class SFGUI_API Signal {
	public:
		/** Ctor.
		 */
		Signal();

		/** Dtor.
		 */
		~Signal();

		/** Connect to free function.
		 * @param delegate Free function.
		 * @return Connection serial, use for disconnecting.
		 */
		unsigned int Connect( Delegate delegate );

		/** Connect to non-static member function.
		 * @param function Function.
		 * @param object Object pointer.
		 * @return Connection serial, use for disconnecting.
		 */
		template <class Class>
		unsigned int Connect( void(Class::*function)(), Class* object );

		/** Disconnect signal.
		 * @param serial Serial generated by Connect().
		 */
		void Disconnect( unsigned int serial );

		/** Emit.
		 */
		void operator()() const;

	private:
		typedef std::map<unsigned int, Delegate> DelegateMap;

		static unsigned int m_serial;
		DelegateMap* m_delegates;
};

}

#include "Signal.inl"
